RMS.LoadLibrary("rmgen");

// initialize map
log("Initializing map...");
InitMap();


// Set some variables
var numPlayers = getNumPlayers();
var mapSize = getMapSize();
var mapRadius = getMapSize()/2;
var mapCenterX = mapRadius;
var mapCenterZ = mapRadius;
var mapCenter = [mapCenterX, mapCenterZ];
var playableMapRadius = mapRadius - 5; // Might be slightly bigger

// Helper functions

// Funtion to place civil centre and starting units at given location for a given player
function placeStartEntities(startLocation, playerId)
// "startLocation" should be a list of length 2 with floats determining the 2d-point where the starting entities shall be placed.
// "startLocation[0]" should be an float determining the x coordinate of the start location.
// "startLocation[1]" should be an float determining the z coordinate of the start location. (Should by y in my book though)
// "playerId" should be the player number like seen in Atlas or the game settings ingame (Starting with 1 for the first player with default colour blue, 0 is Gaia)
{
	// Some checks. Enhance if needed...
	
	// Setting some variables
	// Setting default building angle, can be got from the global "BUILDING_ANGlE" in further versions
	var buildingOrientation = 3*Math.PI/4;
	// Getting civilisation dependent default starting entities
	var civEntities = g_CivData[g_MapSettings.PlayerData[i].Civ].StartEntities;
	
	// Place civil centre on the start location
	placeObject(startLocation[0], startLocation[1], civEntities[0].Template, playerId, buildingOrientation);
	
	// Place starting units
	// Minimal distance to start location so that units don't stand in the civil centere
	var minDistToCC = 4;
	// Minimal distance between units so they don't overlap
	var minDist = 1;
	// Set angle width units should be spread over
	var angleSpread = PI/4;
	// Outer loop for the diffrent unit types to place beside the civil centre (so starting with 1)
	for (var unitTypeIndex = 1; unitTypeIndex < civEntities.length; ++unitTypeIndex)
	{
		// Reset default number of units of that type
		var unitCount = (civEntities[unitTypeIndex].Count !== undefined ? civEntities[unitTypeIndex].Count : 1);
		// Reset the angle added per unit of that type depending on the number of them, for multiple units of that type only (otherwise 0 devision)
		if (unitCount > 1) {var angleAdd = angleSpread/(unitCount-1)};
		// Inner loop for multiple units of the same type
		for (var unitIndex = 0; unitIndex < unitCount; unitIndex++)
		{
			// Resetting the angle the unit will be placed in comperison to the city centre
			if (unitCount > 1) {var angleActual = buildingOrientation - angleSpread/2 + unitIndex*angleAdd}
			else {var angleActual = buildingOrientation};
			// Place the unit
			placeObject(
				startLocation[0] + (minDistToCC + minDist*unitTypeIndex)*sin(angleActual),
				startLocation[1] + (minDistToCC + minDist*unitTypeIndex)*cos(angleActual),
				civEntities[unitTypeIndex].Template, playerId, angleActual);
		};
	};
};

// Vector functions
// Function to get the length of a given vector
function getLength(vector)
{
	return Math.pow(Math.pow(vector[0], 2) + Math.pow(vector[1], 2), 1/2);
};

// Function to get the sum of 2 given vectors
function getSum(v1, v2)
{
	return [v1[0] + v2[0], v1[1] + v2[1]]
};

// Function to get the difference of 2 given vectors
function getDif(v1, v2)
{
	return [v1[0] - v2[0], v1[1] - v2[1]]
};

// Function to get scalar product of 2 vectors
function getDotProduct(v1, v2)
{
	return v1[0]*v2[0] + v1[1]*v2[1];
};

// Function to get the distance between 2 points
function getDistance(point1, point2)
{
	return Math.pow(Math.pow(point1[0] - point2[0], 2) + Math.pow(point1[1] - point2[1], 2), 1/2);
};

// Function to get the direction from one point to another
function getDirection(point1, point2)
{
	var vector = [point2[0] - point1[0], point2[1] - point1[1]];
	var output = 0;
	if (vector[0] !== 0 || vector[1] !== 0)
	{
		var output = Math.acos(vector[0]/getDistance(point1, point2));
		if (vector[1] > 0) {output = PI + (PI - Math.acos(vector[0]/getDistance(point1, point2)))};
		output = (output + PI/2) % (2*PI);
	};
	return output;
};

// Function to get the angle between 2 vectors
function getAngle(v1, v2)
{
	var output = 0;
	if ((v1[0] !== 0 || v1[1] !== 0) && (v2[0] !== 0 || v2[1] !== 0))
	{
		var d1 = getDirection([0, 0], v1);
		var d2 = getDirection([0, 0], v2);
		output = ((d2 - d1) + 2*PI) % (2*PI);
	};
	return output;
};

// Test the functions...
// var vectors = [];
// vectors.push([0, 1]);
// vectors.push([1, 1]);
// vectors.push([1, 0]);
// vectors.push([1, -1]);
// vectors.push([0, -1]);
// vectors.push([-1, -1]);
// vectors.push([-1, 0]);
// vectors.push([-1, 1]);
// vectors.push([0, 0]);
// for (var i = 0; i < vectors.length; i++)
// {
	// for (var j = 0; j < vectors.length; j++)
	// {
		// log("vectors[i] = " + vectors[i]);
		// log("vectors[j] = " + vectors[j]);
		// log("getDirection(vectors[i], vectors[j]) = " + getDirection(vectors[i], vectors[j]));
		// log("getAngle(vectors[i], vectors[j]) = " + getAngle(vectors[i], vectors[j]));
	// };
// };

// Create array of tiles together roughly shaping a circle arround a given coordinate (that should be a tile, too)
function getCircleTiles(center, radius)
{
	if (typeof(center[0]) !== "int" || typeof(center[1]) !== "int") // How to decide if argument is int or float???
	{
		// warn("X and/or Z coordinates are given as float, integers recommended")
		center[0] = round(center[0]);
		center[1] = round(center[1]);
	};
	var result = [center];
	var touch = [[0, 1], [1, 0], [0, -1], [-1, 0]];
	var near = [[0, 1], [1, 1], [1, 0], [1, -1], [0, -1], [-1, -1], [-1, 0], [-1, 1]];
	for (var iRadius = 0; iRadius < radius; iRadius++)
	{
		if (iRadius%2 == 0)
			{var shape = touch}
		else
			{var shape = near};
		var actualResults = result.length;
		for (var iResult = 0; iResult < actualResults; iResult++)
		{
			for (var iShape = 0; iShape < shape.length; iShape++)
			{
				var actualTile = [result[iResult][0]+shape[iShape][0], result[iResult][1]+shape[iShape][1]];
				var inResult = false;
				for (var jResult = 0; jResult < actualResults; jResult++)
				{
					if (actualTile == result[jResult])
					{
						inResult = true;
						// break;
					};
				};
				if (inResult == false)
				{
					result.push(actualTile);
				};
			};
		};
	};
	
	return result;
};
// var circle = getCircleTiles(mapCenter, 5);
// for (var i = 0; i < circle.length; i++)
// {
	// placeTerrain(circle[i][0], circle[i][1], "desert_city_tile_plaza");
// };

// Functionality to create forts/fortresses
// Wall object dictionary
// var palisadesFortress = {};

// var wall = {}; // Straight hard to destroy blocker, a straight wall for exsample
// wall["entity"] = "other/palisades_rocks_straight";
// wall["angle"] = -PI/2; // Orientation so 'outside' = 'top' = '+z' ('inside' = 'bot' = '-z')
// wall["width"] = 2.5; // Length in x direction with default orientation ('outside' = 'top' = '+z')
// wall["indent"] = 0; // The straight wall should always has no indent!
// wall["bending"] = 0*PI; 
// palisadesFortress["wall"] = wall;

// var tower = {}; // A weak defensive structure put inside the wall, a wall tower for exsample
// tower["entity"] = "other/palisades_rocks_tower"; // Tower to put between wall entities
// tower["angle"] = 0*PI; // Oriented so 'outside' = 'top' = '+z'
// tower["width"] = 0.75; // Thickness in x direction with default orientation
// tower["indent"] = 0;
// tower["bending"] = 0*PI; 
// palisadesFortress["tower"] = tower;

// var wallFort = {}; // A strong defensive structure put inside the wall, a fortress for exsample
// wallFort["entity"] = "other/palisades_rocks_fort"; // Fortress that guards entries/gates or even can be placed between wall entities
// wallFort["angle"] = 0*PI; // Oriented so 'front' = 'top' = '+z'
// wallFort["width"] = 1.7; // Thickness in x direction with default orientation
// wallFort["indent"] = 0; // Minimum radius needed so units can pass between it and walls/wall towers/outposts/fortresses
// wallFort["bending"] = 0*PI; 
// palisadesFortress["wallFort"] = wallFort;

// var outpost = {}; // A defensive structure behind the wall not occupying any space along the wall and don't devide it, an outpost for exsample
// outpost["entity"] = "other/palisades_rocks_outpost"; // Tower to put near the entries/gates
// outpost["angle"] = 0*PI; // Oriented so 'front' = 'top' = '+z'
// outpost["width"] = 4; // Should be 0 because it's not part of the wall HAS TO BE REWRITTEN!!!
// outpost["indent"] = 3; // Minimum radius needed so units can pass between it and walls/wall towers/outposts/fortresses
// outpost["bending"] = 0*PI; 
// outpost["wallFort"] = outpost;

// // Entries and gates. So far very similar to straight objects. Have more likely an in- and outsite though.
// var gate = {}; // A structure that can be walked through, a gate for exsample (doesn't work unill now so inside entrys!)
// gate["entity"] = "other/palisades_rocks_gate"; // Gate that should be able to open (not the case untill now so an opened entry should be present)
// gate["angle"] = -PI/2; // Orientes so 'outside' = 'top' = '+z'
// gate["width"] = 3.6; // Length in x direction with default orientation
// gate["indent"] = 0;
// gate["bending"] = 0*PI; 
// outpost["gate"] = gate;

// var entry = {}; // An open entry representes by a single entity, an indented outpost for exsample that splits the wall.

// var wideEntry = {}; // An wide open entry representes by a single entity, an far indentet fortress for exsample


// Wall object
// function wallObject(entity)
// {
	// this.name = "wall"
	// this.entity = "other/palisades_rocks_straight";
	// this.angle = -PI/s;
	// this.width = 2.5;
	// this.indent = 0;
	// this.bending = 0*PI;
// };

// function fortressBuilder()
// {
	// // Straight objects. Doesn't necessarily make the wall longer but does'nt bend them.
	// this.wall = {}; // Straight hard to destroy blocker, a straight wall for exsample
	// this.wall["entity"] = "other/palisades_rocks_straight";
	// this.wall["angle"] = -PI/2; // Orientation so 'outside' = 'top' = '+z' ('inside' = 'bot' = '-z')
	// this.wall["width"] = 2.5; // Length in x direction with default orientation ('outside' = 'top' = '+z')
	// this.wall["indent"] = 0; // The straight wall should always has no indent!
	// this.wall["bending"] = 0*PI; 
	// var wallEntity = "other/palisades_rocks_straight";
	// var wallAngle = -PI/2; // Orientation so 'outside' = 'top' = '+z' ('inside' = 'bot' = '-z')
	// var wallWidth = 2.5; // Length in x direction with default orientation ('outside' = 'top' = '+z')
	// var wallIndent = 0; // The straight wall should always has no indent!
	
	// var tower = {}; // A weak defensive structure to put inside the wall, a wall tower for exsample
	// tower["entity"] = "other/palisades_rocks_tower"; // Tower to put between wall entities
	// tower["angle"] = 0*PI; // Oriented so 'outside' = 'top' = '+z'
	// tower["width"] = 0.75; // Thickness in x direction with default orientation
	// tower["indent"] = 0;
	// tower["bending"] = 0*PI; 
	// var towerEntity = "other/palisades_rocks_tower"; // Tower to put between wall entities
	// var towerAngle = 0*PI; // Oriented so 'outside' = 'top' = '+z'
	// var towerWidth = 0.75; // Thickness in x direction with default orientation
	// var towerIndent = 0;
	
	// var wallFort = {}; // A strong defensive structure put inside the wall, a fortress for exsample
	// wallFort["entity"] = "other/palisades_rocks_fort"; // Fortress that guards entries/gates or even can be placed between wall entities
	// wallFort["angle"] = 0*PI; // Oriented so 'front' = 'top' = '+z'
	// wallFort["width"] = 1.7; // Thickness in x direction with default orientation
	// wallFort["indent"] = 0; // Minimum radius needed so units can pass between it and walls/wall towers/outposts/fortresses
	// wallFort["bending"] = 0*PI; 
	// var fortEntety = "other/palisades_rocks_fort"; // Fortress that guards entries/gates or even can be placed between wall entities
	// var fortAngle = 0*PI; // Oriented so 'front' = 'top' = '+z'
	// var fortWidth = 1.7; // Thickness in x direction with default orientation
	// var fortIndent = 0; // Minimum radius needed so units can pass between it and walls/wall towers/outposts/fortresses
	
	// var outpost = {}; // A defensive structure behind the wall not occupying any space along the wall and don't devide it, an outpost for exsample
	// outpost["entity"] = "other/palisades_rocks_outpost"; // Tower to put near the entries/gates
	// outpost["angle"] = 0*PI; // Oriented so 'front' = 'top' = '+z'
	// outpost["width"] = 4; // Should be 0 because it's not part of the wall HAS TO BE REWRITTEN!!!
	// outpost["indent"] = 3; // Minimum radius needed so units can pass between it and walls/wall towers/outposts/fortresses
	// outpost["bending"] = 0*PI; 
	// var outpostEntity = "other/palisades_rocks_outpost"; // Tower to put near the entries/gates
	// var outpostAngle = 0*PI; // Oriented so 'front' = 'top' = '+z'
	// var outpostWidth = 4; // Should be 0 because it's not part of the wall HAS TO BE REWRITTEN!!!
	// var outpostIndent = 3; // Minimum radius needed so units can pass between it and walls/wall towers/outposts/fortresses
	
	// // Entries and gates. So far very similar to straight objects. Have more likely an in- and outsite though.
	// var gate = {}; // A structure that can be walked through, a gate for exsample (doesn't work unill now so inside entrys!)
	// gate["entity"] = "other/palisades_rocks_gate"; // Gate that should be able to open (not the case untill now so an opened entry should be present)
	// gate["angle"] = -PI/2; // Orientes so 'outside' = 'top' = '+z'
	// gate["width"] = 3.6; // Length in x direction with default orientation
	// gate["indent"] = 0;
	// gate["bending"] = 0*PI; 
	// var gateEntity = "other/palisades_rocks_gate"; // Gate that should be able to open (not the case untill now so an opened entry should be present)
	// var gateAngle = -PI/2; // Orientes so 'outside' = 'top' = '+z'
	// var gateWidth = 3.6; // Length in x direction with default orientation
	// var gateIndent = 0;
	
	// var entry = {}; // An open entry representes by a single entity, an indented outpost for exsample that splits the wall.
	
	// var wideEntry = {}; // An wide open entry representes by a single entity, an far indentet fortress for exsample
// };
// fortressBuilder.prototype.place = function(position, orientation, playerId)
// {
	// // Setting up some variables
	// var entryWidth = 10; // Width of an entry needed so units can pass
	// var fLength = 30; // Target length of the mighty fortress in direction of the orientation
	// var fWidth = 30; // Target width of the mighty fortress at right angle to the orientation
	
	// // Straight objects. Doesn't necessarily make the wall longer but does'nt bend them.
	// var wall = {}; // Straight hard to destroy blocker, a straight wall for exsample
	// wall["entity"] = "other/palisades_rocks_straight";
	// wall["angle"] = -PI/2; // Orientation so 'outside' = 'top' = '+z' ('inside' = 'bot' = '-z')
	// wall["width"] = 2.5; // Length in x direction with default orientation ('outside' = 'top' = '+z')
	// wall["indent"] = 0; // The straight wall should always has no indent!
	// wall["bending"] = 0*PI; 
	// var wallEntity = "other/palisades_rocks_straight";
	// var wallAngle = -PI/2; // Orientation so 'outside' = 'top' = '+z' ('inside' = 'bot' = '-z')
	// var wallWidth = 2.5; // Length in x direction with default orientation ('outside' = 'top' = '+z')
	// var wallIndent = 0; // The straight wall should always has no indent!
	
	// var tower = {}; // A weak defensive structure put inside the wall, a wall tower for exsample
	// tower["entity"] = "other/palisades_rocks_tower"; // Tower to put between wall entities
	// tower["angle"] = 0*PI; // Oriented so 'outside' = 'top' = '+z'
	// tower["width"] = 0.75; // Thickness in x direction with default orientation
	// tower["indent"] = 0;
	// tower["bending"] = 0*PI; 
	// var towerEntity = "other/palisades_rocks_tower"; // Tower to put between wall entities
	// var towerAngle = 0*PI; // Oriented so 'outside' = 'top' = '+z'
	// var towerWidth = 0.75; // Thickness in x direction with default orientation
	// var towerIndent = 0;
	
	// var wallFort = {}; // A strong defensive structure put inside the wall, a fortress for exsample
	// wallFort["entity"] = "other/palisades_rocks_fort"; // Fortress that guards entries/gates or even can be placed between wall entities
	// wallFort["angle"] = 0*PI; // Oriented so 'front' = 'top' = '+z'
	// wallFort["width"] = 1.7; // Thickness in x direction with default orientation
	// wallFort["indent"] = 0; // Minimum radius needed so units can pass between it and walls/wall towers/outposts/fortresses
	// wallFort["bending"] = 0*PI; 
	// var fortEntety = "other/palisades_rocks_fort"; // Fortress that guards entries/gates or even can be placed between wall entities
	// var fortAngle = 0*PI; // Oriented so 'front' = 'top' = '+z'
	// var fortWidth = 1.7; // Thickness in x direction with default orientation
	// var fortIndent = 0; // Minimum radius needed so units can pass between it and walls/wall towers/outposts/fortresses
	
	// var outpost = {}; // A defensive structure behind the wall not occupying any space along the wall and don't devide it, an outpost for exsample
	// outpost["entity"] = "other/palisades_rocks_outpost"; // Tower to put near the entries/gates
	// outpost["angle"] = 0*PI; // Oriented so 'front' = 'top' = '+z'
	// outpost["width"] = 4; // Should be 0 because it's not part of the wall HAS TO BE REWRITTEN!!!
	// outpost["indent"] = 3; // Minimum radius needed so units can pass between it and walls/wall towers/outposts/fortresses
	// outpost["bending"] = 0*PI; 
	// var outpostEntity = "other/palisades_rocks_outpost"; // Tower to put near the entries/gates
	// var outpostAngle = 0*PI; // Oriented so 'front' = 'top' = '+z'
	// var outpostWidth = 4; // Should be 0 because it's not part of the wall HAS TO BE REWRITTEN!!!
	// var outpostIndent = 3; // Minimum radius needed so units can pass between it and walls/wall towers/outposts/fortresses
	
	// // Entries and gates. So far very similar to straight objects. Have more likely an in- and outsite though.
	// var gate = {}; // A structure that can be walked through, a gate for exsample (doesn't work unill now so inside entrys!)
	// gate["entity"] = "other/palisades_rocks_gate"; // Gate that should be able to open (not the case untill now so an opened entry should be present)
	// gate["angle"] = -PI/2; // Orientes so 'outside' = 'top' = '+z'
	// gate["width"] = 3.6; // Length in x direction with default orientation
	// gate["indent"] = 0;
	// gate["bending"] = 0*PI; 
	// var gateEntity = "other/palisades_rocks_gate"; // Gate that should be able to open (not the case untill now so an opened entry should be present)
	// var gateAngle = -PI/2; // Orientes so 'outside' = 'top' = '+z'
	// var gateWidth = 3.6; // Length in x direction with default orientation
	// var gateIndent = 0;
	
	// var entry = {}; // An open entry representes by a single entity, an indented outpost for exsample that splits the wall.
	
	// var wideEntry = {}; // An wide open entry representes by a single entity, an far indentet fortress for exsample
	
	// // Wall ends. NOTE: The function doesn't end here if more objects stacked!
	// var endRight = {}; // A structure marking the end of a wall, a tower or the palisades end. Right, when you stand uppon the wall looking outside.
	// endRight["entity"] = "other/palisades_rocks_gate"; // Gate that should be able to open (not the case untill now so an opened entry should be present)
	// endRight["angle"] = -PI/2; // Orientes so 'outside' = 'top' = '+z'
	// endRight["width"] = 3.6; // Length in x direction with default orientation
	// endRight["indent"] = 0;
	// endRight["bending"] = 0*PI; 
	// var endEntity = "other/palisades_rocks_end";
	// var endAngle = -PI/2; // Oriented so it fits to a wall to the right, 'outside' = 'top', ending right
	// var endWidth = 1; // Length in x direction whith default orientation (for entry width)
	// var endIndent = 0;
	
	// var endLeft = {}; // A structure marking the end of a wall, a tower or the palisades end.  Left, when you stand uppon the wall looking outside.
	// endLeft["entity"] = "other/palisades_rocks_gate"; // Gate that should be able to open (not the case untill now so an opened entry should be present)
	// endLeft["angle"] = -PI/2; // Orientes so 'outside' = 'top' = '+z'
	// endLeft["width"] = 3.6; // Length in x direction with default orientation
	// endLeft["indent"] = 0;
	// endLeft["bending"] = 0*PI; 
	// var endEntity = "other/palisades_rocks_end";
	// var endAngle = -PI/2; // Oriented so it fits to a wall to the right, 'outside' = 'top', ending right
	// var endWidth = 1; // Length in x direction whith default orientation (for entry width)
	// var endIndent = 0;
	
	// // Corners. All kinds of objects bending the wall.
	// var cornerIn = {}; // Wall corner bending into the fortress (if available or tower)
	// cornerIn["entity"] = "other/palisades_rocks_curve";
	// cornerIn["angle"] = -PI/4; // Orientation so it fits to walls to the right, 'outside' = 'top right', bending bottom
	// cornerIn["width"] = 2; // Distance (x, left/right) with default orientation from left to right end (point of connection) NOT THE DISTANCE!
	// cornerIn["indent"] = 0;
	// cornerIn["bending"] = PI/2; 
	// var cornerEntity = "other/palisades_rocks_curve";
	// var cornerAngle = -PI/4; // Orientation so it fits to walls to the right, 'outside' = 'top right', bending bottom
	// var cornerWidth = 2; // Distance (x, left/right) with default orientation from left to right end (point of connection) NOT THE DISTANCE!
	// var cornerIndent = 0;
	
	// var cornerOut = {}; // Wall corner bending out of the fortress (if available or tower)
	
	// var MfTypeList = []; //List of fortress types. Type 0 should be build automatically to fit in a circle of a given radius. Others might stay fixed.
	// MfTypeList[1] = [];
	
	// // Actually placing the mighty fortress
	// var repeatList = [];
	// var iterEntList = [towerEntity, wallEntity, fortEntety, wallEntity, towerEntity, wallEntity, outpostEntity, wallEntity, towerEntity, wallEntity, fortEntety, wallEntity, towerEntity];
	// var iterAngleList = [towerAngle, wallAngle, fortAngle, wallAngle, towerAngle, wallAngle, outpostAngle, wallAngle, towerAngle, wallAngle, fortAngle, wallAngle, towerAngle];
	// var iterWidthList = [towerWidth, wallWidth, fortWidth, wallWidth, towerWidth, wallWidth, outpostWidth, wallWidth, towerWidth, wallWidth, fortWidth, wallWidth, towerWidth];
	// var iterIndentList = [towerIndent, wallIndent, fortIndent, wallIndent, towerIndent, wallIndent, outpostIndent, wallIndent, towerIndent, wallIndent, fortIndent, wallIndent, towerIndent];
	// var x = position[0];
	// var z = position[1];
	// for (var iEnt = 0; iEnt < iterEntList.length; iEnt++)
	// {
		// if (iEnt !== 0)
		// {
			// var actualZ = z - iterIndentList[iEnt];
			// placeObject(x, actualZ, iterEntList[iEnt], playerId, iterAngleList[iEnt]);
			// x += (iterWidthList[iEnt] + iterWidthList[(iEnt + 1)%iterWidthList.length])/2;
		// };
	// };
// };
// var myFortress = new fortressBuilder;
// myFortress.place(mapCenter, 0*PI, 1);

// create tile classes

// var clPlayer = createTileClass();
// var clPath = createTileClass();
// var clHill = createTileClass();
// var clForest = createTileClass();
// var clWater = createTileClass();
// var clRock = createTileClass();
// var clFood = createTileClass();
// var clBaseResource = createTileClass();

var tWood = ["forestfloor_dirty", "forestfloor_pine"];
var tPath = ["road_muddy", "temp_road_muddy"]; // "temp_road_muddy", "road_muddy", "temp_road", "temp_road_overgrown"
var tPathBorder = ["mud_slide_2", "medit_grass_wild", "medit_grass_wild", "medit_grass_wild", "new_alpine_grass_a", "new_alpine_grass_a"]; // "medit_grass_wild", "mud_slide_2", "new_alpine_grass_a", "new_alpine_grass_mossy"
var tBase = ["mud_slide_2", "new_alpine_grass_a", "grass_field", "grass_field_b", "medit_grass_field"];

// Setting up players start position
var playerStartLocation = new Array(numPlayers);
var playerAngle = new Array(numPlayers);
var playerBuildingAngle = new Array(numPlayers);
var baseRadius = 25; // Space needed to build buildings
var minR = 0.6 * mapRadius;
var maxR = 0.75 * mapRadius;
var startAngle = randFloat(0, 2*PI);
var angleDist = 2*PI/numPlayers;
var angleVar = 0.15*angleDist;
for (var i=0; i < numPlayers; i++)
{
	playerAngle[i] = (i*angleDist + startAngle + randFloat(-angleVar, angleVar))%(2*PI);
	if (playerAngle[i] <= PI/2) // top right
	{playerBuildingAngle[i] = 5*PI/4} // face bottom left
	else
	{
		if (playerAngle[i] <= PI) // top left
		{playerBuildingAngle[i] = 3*PI/4} // face bottom right
		else
		{
			if (playerAngle[i] <= 3*PI/2) // bottom left
			{playerBuildingAngle[i] = PI/4} // face top right
			else
			{
				if (playerAngle[i] <= 2*PI) // bottom right
				{playerBuildingAngle[i] = 7*PI/4} // face top left
				else
				{
					playerBuildingAngle[i] = 3*PI/4;
					warn("Player" + i + "'s building angle was not set properly, his angle %(2*PI) (from center):" + playerAngle[i]);
				};
			};
		};
	};
	var x = mapCenterX + randFloat(minR, maxR)*cos(playerAngle[i]);
	var z = mapCenterZ + randFloat(minR, maxR)*sin(playerAngle[i]);
	playerStartLocation[i] = [x, z];
};


// Place starting entities
for (var i=0; i < numPlayers; i++)
	{placeStartEntities(playerStartLocation[i], i+1)};


// Place resources at start locations
for (var i=0; i < numPlayers; i++)
{
	// Start location recources
	var distToCC = 10;
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]);
	placeObject(x, z, "gaia/flora_tree_baobab", 0, randFloat(0, 2*PI));
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]+PI/8);
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]+PI/8);
	placeObject(x, z, "gaia/flora_tree_baobab", 0, randFloat(0, 2*PI));
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]-1*PI/8);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]-1*PI/8);
	placeObject(x, z, "gaia/flora_tree_baobab", 0, randFloat(0, 2*PI));
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]+PI/2);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]+PI/2);
	placeObject(x, z, "gaia/geology_metal_mediterranean_slabs", 0, randFloat(0, 2*PI));
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]+3*PI/2);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]+3*PI/2);
	placeObject(x, z, "gaia/geology_stonemine_medit_quarry", 0, randFloat(0, 2*PI));
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]+PI);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]+PI);
	placeObject(x, z, "gaia/flora_bush_berry", 0, randFloat(0, 2*PI));
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]+9*PI/8);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]+9*PI/8);
	placeObject(x, z, "gaia/flora_bush_berry", 0, randFloat(0, 2*PI));
	var x = playerStartLocation[i][0] + distToCC*cos(playerAngle[i]+7*PI/8);
	var z = playerStartLocation[i][1] + distToCC*sin(playerAngle[i]+7*PI/8);
	placeObject(x, z, "gaia/flora_bush_berry", 0, randFloat(0, 2*PI));
};


// Paths
// Paths between start positions
var minDistanceToSP = 10;
var minThickness = 1;
var maxThickness = 1;
var maxAngleOffset = PI/2;
var density = 0.5; // 1 means about 1 tile between "path spots", 2 means 1/2 tile, 1/2 means 2...
var maintainance = 0.9;
var pathSinking = 0.3;
var bumpyness = 0.1;
var maxI = numPlayers;
if (numPlayers % 2 == 1)
{
	maxI = numPlayers + 1;
};
for (var i = 0; i < maxI; i++)
{
	var minJ = i+1;
	if (numPlayers < 5)
	{
		minJ = 0;
	};
	for (var j = minJ; j < numPlayers + 1; j++) // Set j = i+1 for 1 road between points, 0 for 2 roads
	{
		var start = mapCenter;
		var target = mapCenter;
		if (i < numPlayers)
			{var start = playerStartLocation[i]};
		if (j < numPlayers)
			{var target = playerStartLocation[j]};
		var ix = round(start[0] + minDistanceToSP*sin(getDirection(start, target)));
		var iz = round(start[1] + minDistanceToSP*cos(getDirection(start, target)));
		var distToTarget = minDistanceToSP + 1; // Only preset here...
		var trys = 0;
		while (distToTarget > minDistanceToSP && trys < 1000 && getDistance(mapCenter, [ix, iz]) + maxThickness + 1 < mapRadius)
		{
			var thickness = randInt(minThickness, maxThickness);
			var pathTiles = getCircleTiles([ix, iz], thickness);
			var borderTiles = getCircleTiles([ix, iz], thickness + 1);
			for (var k = 0; k < pathTiles.length; k++)
			{
				if (randFloat() > bumpyness)
				{
					setHeight(pathTiles[k][0], pathTiles[k][1], getHeight(pathTiles[k][0], pathTiles[k][1]) - pathSinking);
				};
				if (randFloat() < maintainance)
				{
					placeTerrain(pathTiles[k][0], pathTiles[k][1], tPath);
				};
			};
			for (var k = 0; k < borderTiles.length; k++)
			{
				var placementOk = true;
				for (var ti = 0; ti < tPath.length; ti++)
				{
					if (g_Map.getTexture(borderTiles[k][0], borderTiles[k][1]) == tPath[ti])
					{
						placementOk = false;
					};
				};
				if (placementOk)
				{
					placeTerrain(borderTiles[k][0], borderTiles[k][1], tPathBorder);
				};
			};
			// Set parameters for next loop
			ix = round(ix + sin(getDirection([ix, iz], target) + randFloat(-3*maxAngleOffset/2, maxAngleOffset/2))/density);
			iz = round(iz + cos(getDirection([ix, iz], target) + randFloat(-3*maxAngleOffset/2, maxAngleOffset/2))/density);
			if (j-i > 0)
			{
				ix = round(ix + sin(getDirection([ix, iz], target) + randFloat(-maxAngleOffset/2, 3*maxAngleOffset/2))/density);
				iz = round(iz + cos(getDirection([ix, iz], target) + randFloat(-maxAngleOffset/2, 3*maxAngleOffset/2))/density);
			};
			distToTarget = getDistance([ix, iz], target);
			trys++
		};
	};
};


// General hight map
var maxHeight = 0.2 * mapRadius;
var minHeight = 1;
var maxHeightOff = 0.5;
var maxHillRadius = 0.6 * mapRadius;
// Lakes
var lakeTexture = ["dirt_beach", "dirt_pebbles_wet"];
var minWaterRadius = 0.80 * mapRadius;
var minWaterDistToSP = baseRadius + 15;
var maxLakeHeightOff = 0.9;
var maxLakeTextureHeight = 0.2;
// Central eye candy (No, not a pyramid ^^)
var centerSpace = 20;
var eyeCandy = "other/unfinished_greek_temple";
placeObject(mapRadius, mapRadius, eyeCandy, 0, randFloat(0, 2*PI));
// Preset Woods
var maxTreeDensity = 0.3;
var minForestfloorRadius = 2;
var maxForestfloorRadius = 3;
var forestfloorDensity = 0.2;
var forestfloorHeightIncrease = 0.1;
var forestfloorHeightDensity = 0.5;
var minTreeGroundHeight = 0.8;
var treeFullDensityRadiusToSP = baseRadius + 10;
// Preset Bases
var basefloorRadius = baseRadius - 5;
var maxBasefloorDensity = 0.75;
var maxBasefloorRaise = 0.5;
// Additional Recources
var recourceRadius = mapRadius/2;
var resourcePerPlayer = 3;
var metalTemplate = "gaia/geology_metal_mediterranean_slabs";
var stoneTemplate = "gaia/geology_stonemine_medit_quarry";
var metalProb = 1; // stoneProb = 1 - metalProb
for (var playerId = 0; playerId < numPlayers; playerId++)
{
	var actualAngle = playerAngle[playerId];
	var v1 = [playerStartLocation[playerId][0] - mapCenter[0], playerStartLocation[playerId][1] - mapCenter[1]];
	var v2 = [playerStartLocation[(playerId+1)%numPlayers][0] - mapCenter[0], playerStartLocation[(playerId+1)%numPlayers][1] - mapCenter[1]];
	var angleDistPlayer = getAngle(v2, v1);
	var angleDist = angleDistPlayer / (resourcePerPlayer + 1)
	for (var iResource = 0; iResource < resourcePerPlayer; iResource++)
	{
		actualAngle += angleDist;
		var ix = round(mapCenter[0] + recourceRadius*cos(actualAngle));
		var iz = round(mapCenter[0] + recourceRadius*sin(actualAngle));
		if (randFloat() < metalProb)
			{placeObject(ix, iz, metalTemplate, 0, randFloat(0, 2*PI))}
		else
			{placeObject(ix, iz, stoneTemplate, 0, randFloat(0, 2*PI))};
	};
};

// Building map tile by tile
for (var ix = 0; ix < mapSize; ix++)
{
	for (var iz = 0; iz < mapSize; iz++)
	{
		// General variables
		var actualRadius = getDistance([ix, iz], mapCenter);
		var distToNextSP = mapRadius;
		for (var playerId = 0; playerId < numPlayers; playerId++)
		{
			var distToSP = getDistance([ix, iz], playerStartLocation[playerId]);
			if (distToSP < distToNextSP)
			{
				distToNextSP = distToSP;
			};
		};
		// General hight map
		if (actualRadius < mapRadius)
		{
			var heightDerivation = minHeight;
			if (actualRadius < maxHillRadius)
			{
				// var heightDerivationX = cos(actualRadius * PI/2 /maxHillRadius);
				// var heightDerivationZ = cos(actualRadius * PI/2 /maxHillRadius);
				var heightDerivation = ((maxHeight-minHeight) * cos(actualRadius * PI/2 /maxHillRadius)) + minHeight;
			};
			setHeight(ix, iz, getHeight(ix, iz) + heightDerivation + randFloat(-maxHeightOff, maxHeightOff))
		};
		// Lakes
		if (actualRadius > minWaterRadius && distToNextSP > minWaterDistToSP)
		{
			var maxDepthBySP = distToNextSP - minWaterDistToSP;
			var maxDepthByRadius = actualRadius - minWaterRadius;
			var lakeHightDerivation = -min(maxDepthBySP, maxDepthByRadius);
			setHeight(ix, iz, getHeight(ix, iz) + lakeHightDerivation + randFloat(-maxLakeHeightOff, maxLakeHeightOff));
			if (getHeight(ix, iz) < maxLakeTextureHeight)
			{
				placeTerrain(ix, iz, lakeTexture)
			};
		};
		if (actualRadius < playableMapRadius)
		{
			// Woods
			var treeDerivation = maxTreeDensity * (getDistance(mapCenter, [ix, iz])-mapRadius/2)/(mapRadius/2);
			var treeDensity = max(treeDerivation, -treeDerivation);
			var densityByDistToSP = min((distToNextSP - baseRadius)/(treeFullDensityRadiusToSP - baseRadius), 1);
			treeDensity = treeDensity * densityByDistToSP;
			var placementOk = true;
			for (var texId = 0; texId < tPath.length; texId++)
			{
				if (g_Map.getTexture(ix, iz) == tPath[texId])
					{placementOk = false};
			};
			if (getHeight(ix, iz) < minTreeGroundHeight)
			{
				placementOk = false;
			};
			if (distToNextSP < baseRadius)
			{
				placementOk = false;
			};
			if (getDistance(mapCenter, [ix, iz]) < centerSpace)
			{
				placementOk = false;
			};
			for (var texId = 0; texId < tPathBorder.length; texId++)
			{
				if (g_Map.getTexture(ix, iz) == tPathBorder[texId])
				{
					placementOk = false;
				};
			};
			if (randFloat() < treeDensity && placementOk)
			{
				var radius = randInt(minForestfloorRadius, maxForestfloorRadius);
				var forestTiles = getCircleTiles([ix, iz], radius);
				for (var it = 0; it < forestTiles.length; it++)
				{
					var tx = forestTiles[it][0];
					var tz = forestTiles[it][1];
					placementOk = true;
					for (var texId = 0; texId < tPath.length; texId++)
					{
						if (g_Map.getTexture(tx, tz) == tPath[texId])
							{placementOk = false};
					};
					for (var texId = 0; texId < tPathBorder.length; texId++)
					{
						if (g_Map.getTexture(tx, tz) == tPathBorder[texId])
							{placementOk = false};
					};

					if (randFloat() < forestfloorDensity && placementOk)
						{placeTerrain(tx, tz, tWood)};
					if (randFloat() < forestfloorHeightDensity)
						{setHeight(tx, tz, getHeight(tx, tz) + forestfloorHeightIncrease)};
				};
				placeObject(ix, iz, "gaia/flora_tree_baobab", 0, randFloat(0, 2*PI));
			};
			// Base
			for (var playerId = 0; playerId < numPlayers;playerId++)
			{
				var distToSP = getDistance([ix, iz], playerStartLocation[playerId]);
				if (distToSP < basefloorRadius)
				{
					if (randFloat() < maxBasefloorDensity*(1-distToSP/basefloorRadius))
					{
						placeTerrain(ix, iz, tBase);
					};
					setHeight(ix, iz, getHeight(ix, iz) + randFloat(0, maxBasefloorRaise));
				};
			};
		};
	};
};


// Additional recources

// Recources in a circle
// var radius = 0.5 * mapRadius;
// var probStone = 0.25; // 0,5 means equally mettal and stone derivation, 0,25 means 3 times the mettal, 0,75 means 3 times the stone
// var numRecources = numPlayers * 3;
// var startAngle = randFloat(0, 2*PI);
// var addedAngle = 2*PI / numRecources;
// for (var i = 0; i < numRecources; i++)
// {
	// var actualAngle = startAngle + i*addedAngle;
	// var x = mapCenter[0] + radius*sin(actualAngle);
	// var z = mapCenter[1] + radius*cos(actualAngle);
	// if (randFloat() < probStone) {placeObject(x, z, "gaia/geology_stonemine_medit_quarry", 0, randFloat(0, 2*PI))}
	// else {placeObject(x, z, "gaia/geology_metal_mediterranean_slabs", 0, randFloat(0, 2*PI))};
// };


// Export map data
ExportMap();
